<?php

/* 
Plugin Name: Nofollow Case by Case
Plugin URI: http://www.fob-marketing.de/marketing-blog-184-wordpress-nofollow-seo-plugin-nofollow-case-by-case.html 
Description: Nofollow Case by Case is a follow (nofollow and dofollow) plugin for WordPress. It allows you to selectively apply nofollow to your comments.
Version: 1.4
Author: Oliver Bockelmann (fob marketing, Hamburg)
Author URI: http://www.fob-marketing.de/
Update Server: http://wordpress.org/extend/plugins/nofollow-case-by-case/
License: Compatible with the GPL2
*/

/*
Copyright 2007 - 2013 Oliver Bockelmann (email : fob@fob-marketing.de)

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License, version 2, as published by the Free Software Foundation. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

/* NEW VERSION SINCE 2013-01-15 **************************************************************************************************************************************** */

// Nofollow and other things to be removed
$diverse_nofollow_variants = array(' rel="nofollow"', " rel='nofollow'", ' rel="external"', " rel='external'", ' rel="external nofollow"', " rel='external nofollow'", ' rel="nofollow external"', ' rel="nofollow external"', ' target="_blank"', " target='_blank'");


/* START INPUT FILTERS */

function nfcbc_wp_rel_follow( $text ) {
	// This is a pre save filter, so text is already escaped.
	$text = stripslashes($text);
	$text = preg_replace_callback('|<a (.+?)>|i', 'nfcbc_wp_rel_follow_callback', $text);
	$text = esc_sql($text);
	return $text;
}

function nfcbc_wp_rel_follow_callback( $matches ) {
	global $diverse_nofollow_variants;
	$text = $matches[1];
	$text = str_replace($diverse_nofollow_variants, '', $text);
	// Now the original $text is nofollow free and can be fitted with other stuff
	return "<a $text>";
}


/* START OUTPUT FILTERS */

// Clean up the comment text
function nfcbc_clean_up_make_clickable( $r ) {
global $diverse_nofollow_variants;
	
	if ( empty($r) ) { $r = $ret; }

	// Remove nofollow from links in comments (all cases, perhaps we clean up themes here and add nofollow just where it seems to be necessary)
	$r = str_replace($diverse_nofollow_variants, "", $r);

	// Insert nofollow where dontfollow is used (case sensitive replacement, standard and preferred typing first)
	$r = str_replace(array('/dontfollow"', "/dontfollow'"), array('" rel="external nofollow"', "' rel='external nofollow'"), $r);

	// Insert rel="external" where we have an external URL not modified yet.
	// Editor should be as clean as possible. No need to show rel="external" there.
	if (!is_admin()){

		// Pick the links from the given comment
		$nfcbc_link_hits = preg_match_all('/<a (.+?)>/', $r, $linkpartmatches);

		// Do we have to do anything else here?
		if ($nfcbc_link_hits) {

			// How many links do we have here?
			// echo 'Hits:' . $nfcbc_link_hits;
			// Our own links are internal not external!
			$nfcbc_check_internal_link = get_home_url();

			for ($nfcbc_x = 0; $nfcbc_x < $nfcbc_link_hits; $nfcbc_x++) {
				// Do not replace our internal links or modified nofollow links!
				if 	( strpos($linkpartmatches[1][$nfcbc_x], '/dontfollow') === false && 
					  strpos($linkpartmatches[1][$nfcbc_x], 'external nofollow') === false && 
					  strpos($linkpartmatches[1][$nfcbc_x], $nfcbc_check_internal_link) === false ) {

					// Just for testing purposes: 
					// echo '<br />Links to be modified:<br />' . $linkpartmatches[1][$nfcbc_x] . '<br />';

					$new_nfcbc_link = $linkpartmatches[1][$nfcbc_x].' rel="external"';

					// Just for testing purposes:
					// echo $linkpartmatches[1][$nfcbc_x]. ' has been modified! <br />';

					// Save the results
					$r = str_replace($linkpartmatches[1][$nfcbc_x], $new_nfcbc_link, $r);

				}
			}
		}
	}

	return $r;
}

function nfcbc_clean_up_author_links( $return ) {
global $diverse_nofollow_variants;

	// Remove nofollow from links in comments (all cases, perhaps we clean up themes here and add nofollow just where it seems to be necessary)
	$return = str_replace($diverse_nofollow_variants, '', $return);

	// Strip dontfollow from the links. 
	// Take care on comment_author_url_link which should not have a /dontfollow linktext!
	$return = str_replace( array('/dontfollow"', "/dontfollow'", '//dontfollow</a>', '/dontfollow</a>'), 
		array('" rel="external nofollow"', "' rel='external nofollow'", '</a>', '</a>'), $return);

	// Insert rel="external" where we have an external URL not modified yet.
	if (!is_admin()){

		// Pick the link from the given comment.
		// preg_match causes problems. So we stay with preg_match_all here.
		$nfcbc_link_hit = preg_match_all('/<a (.+?)>/', $return, $linkpartmatch);

		// Do we have to do anything else here?
		if ($nfcbc_link_hit) {

			// How many links do we have here?
			// echo 'Hits:' . $nfcbc_link_hit;
			// Our own links are internal not external!
			$nfcbc_check_internal_link = get_home_url();

			// Do not replace our internal links or modified nofollow links!
			if 	( strpos($linkpartmatch[1][0], '/dontfollow') === false && 
				  strpos($linkpartmatch[1][0], 'external nofollow') === false && 
				  strpos($linkpartmatch[1][0], $nfcbc_check_internal_link) === false ) {

				// Just for testing purposes: 
				// echo '<br />Links to be modified:<br />' . $linkpartmatch[1][0] . '<br />';

				$new_nfcbc_link = $linkpartmatch[1][0].' rel="external"';

				// Just for testing purposes:
				// echo $linkpartmatch[1][0]. ' has been modified! <br />';

				// Save the results
				$return = str_replace($linkpartmatch[1][0], $new_nfcbc_link, $return);
			}
		}
	}
	return $return;
}


// Modify pre save function
remove_filter('pre_comment_content', 'wp_rel_nofollow', 16);
add_filter('pre_comment_content', 'nfcbc_wp_rel_follow', 17); // Filter for the database entries

// Optimize the output
add_filter('comment_text','nfcbc_clean_up_make_clickable', 18); // Output filter for the comment text links
add_filter('get_comment_author_link','nfcbc_clean_up_author_links', 19); // Output filter for the normal comment author link (depends on template functions used)
add_filter('get_comment_author_url_link', 'nfcbc_clean_up_author_links', 19); // Output filter for further comment author links (depends on template functions used)


// Use XHTML valid version of target="_blank" plus output cleaning option
function nfcbc_open_external_links_script() {
	// Register external scripts after load (footer option). 
	// Includes new window for external links and /dofollow output cleaning for accidential (author) links that can not be modified with PHP on the fly.
	wp_register_script('nfcbc_external_links_script', plugins_url('/js/nfcbc_scripts.js', __FILE__), array('jquery'), '0.1', true);

	// Include this script on pages with comments (only)
	if ( !is_admin() && !is_archive() && get_comments_number() > 0 ) {
		wp_enqueue_script('nfcbc_external_links_script');
	}
}

add_action('wp_enqueue_scripts', 'nfcbc_open_external_links_script'); // This is an XHTML valid target=_blank option
?>